'''
Here we let p be the type B parameter, and q the type A parameter for B_3
and we consider the inclusion A_2 \subset B_3
'''
R.<p,q> = PolynomialRing(ZZ, 2)
HB3 = IwahoriHeckeAlgebra('B3', 1, -q, R)
T = HB3.T()
B3 = WeylGroup('B3')
A2 = WeylGroup('A2')
w0B3 = B3.w0
w0A2 = B3.from_reduced_word(A2.w0.reduced_word())
w0A1A1 = B3.from_reduced_word([1,3])
jelt = w0B3*w0A2
jelt_A1A1 = w0B3*w0A1A1
JM_redex = jelt.reduced_word()
JM_redex_A1A1 = jelt_A1A1.reduced_word()
print "JMA2_redex = " + str(JM_redex)
print "its length = " + str(len(JM_redex))
print "JMA_redex_A1A1 = " + str(JM_redex_A1A1)
print "its length = " + str(len(JM_redex_A1A1))

def T_elt_from_list(expr):
    result = 1
    for index in expr:
        result = result*T[index]
    return result    

def add_term(dic, key, value):
    if key in dic:
        dic[key] += value
    else:
        dic[key] = value

#returns a dictionary whose keys are the w so that T_w appears in T[redex_1]*T[redex_2] and whose values are the corresponding coefficients
def coef_dictionary(redex_1, redex_2):
    result = {}
    result[B3.from_reduced_word(redex_1)] = 1
    redex_2_length = len(redex_2)
    for i in range(redex_2_length):
        print 'iteration' + str(i) + 'of' + str(redex_2_length)
        temp = {}
        j = 0
        result_length = len(result)
        for elt in result:
            j +=1
            print 'calculation' + str(j) + 'of' + str(result_length)
            prod_elt = B3.simple_reflection(redex_2[i])*elt
            if prod_elt.length() > elt.length():
                add_term(temp, prod_elt, result[elt])
            elif redex_2[i] == 3:#here 3 depends on being inside B_3
                add_term(temp, prod_elt, p*result[elt])
                add_term(temp, elt, (1 - p)*result[elt])
            else:
                add_term(temp, prod_elt, q*result[elt])
                add_term(temp, elt, (1 - q)*result[elt])
        result.clear()
        result = temp
    return result

def reduce_to_A2(dic):
    result = {}
    for elt in A2:
        elt = B3.from_reduced_word(elt.reduced_word())
        prod_elt = jelt*elt
        if prod_elt in dic:
            result[elt] = dic[prod_elt]
    return result

#works better if dic is smaller than A2
def faster_reduce_to_A2(dic):
    result = {}
    for elt in dic:
        translated_elt = elt*jelt
        redex = translated_elt.reduced_word()
        if 3 not in redex:#here 3 depends on being inside B_3
            result[translated_elt] = dic[elt]
    return result

def faster_reduce_to_A1A1(dic):
    result = {}
    for elt in dic:
        translated_elt = elt*jelt_A1A1
        redex = translated_elt.reduced_word()
        if 2 not in redex:#here 2 depends on being inside B_3
            result[translated_elt] = dic[elt]
    return result

#removes all key, value pairs such that key has length less than min_length
def trim(dic, min_length):
    print 'trimming: initial size ' + str(len(dic))
    new_dic = {}
    for elt in dic:
        if elt.length() >= min_length:
            new_dic[elt] = dic[elt]
    dic = new_dic
    print 'after trimming: final size ' + str(len(dic))
    return dic

#same as coef_dictionary, but after the half way point every 1 in freq right
#multiplications is followed by a “trimming”, meaning terms which are too short
#are deleted from the dictionary.  Trimming is a bit expensive, so it’s not clear
#what value of freq will create the best (or any) savings
def coef_dictionary_with_trimming(redex_1, redex_2, freq):
    result = {}
    result[B3.from_reduced_word(redex_1)] = 1
    redex_2_length = len(redex_2)
    for i in range(redex_2_length):
        print 'iteration' + str(i) + 'of' + str(redex_2_length)
        temp = {}
        result_length = len(result)
        print 'calculation of size ' + str(result_length) + 'coming'
        for elt in result:
            prod_elt = B3.simple_reflection(redex_2[i])*elt
            if prod_elt.length() > elt.length():
                add_term(temp, prod_elt, result[elt])
            elif redex_2[i] == 3:#here 3 depends on being inside B_3
                add_term(temp, prod_elt, p*result[elt])
                add_term(temp, elt, (1 - p)*result[elt])
            else:
                add_term(temp, prod_elt, q*result[elt])
                add_term(temp, elt, (1 - q)*result[elt])
        result.clear()
        if (i > (redex_2_length + 1)/2 and i % freq == 0) or i > 20:
            temp = trim(temp, i)#redex_2_length - i calcs left, so only include length >=i
        result = temp
    return result
